import math
import random
import time

import pytest

from emtest import EmTest


def sin_waveform(samples: int = 32) -> [float]:
    return [math.cos((i / samples) * 2 * math.pi) * 0.5 + 0.5 for i in range(samples)]


class TestADC(EmTest):

    def setup_class(self):
        self.week_branch = "Week_05"
        super().setup_class(self)

    def set_and_test_values(self, red: float, green: float):
        self.analog.output[0].set_value(green)
        self.analog.output[1].set_value(red)
        time.sleep(0.2)
        assert self.leds.green() == pytest.approx(green, abs=0.25)
        assert self.leds.red() == pytest.approx(red, abs=0.25)

    def test_x_coordinate(self):
        waveform = sin_waveform(32)
        self.analog.output[1].set_value(0)
        for i in waveform:
            self.analog.output[0].set_value(i)
            time.sleep(0.2)
            assert self.leds.green() == pytest.approx(i, abs=0.25)

    def test_y_coordinate(self):
        waveform = sin_waveform(32)
        self.analog.output[0].set_value(0)
        for i in waveform:
            self.analog.output[1].set_value(i)
            time.sleep(0.2)
            assert self.leds.red() == pytest.approx(i, abs=0.25)

    def test_both_coordinates(self):
        waveform = sin_waveform(64)
        for i in waveform:
            self.set_and_test_values(i, 1 - i)

    def test_random_values(self):
        random.seed(42)
        for _ in range(64):
            green = random.uniform(0.0, 1.0)
            red = random.uniform(0.0, 1.0)
            self.set_and_test_values(red, green)
