from typing import Callable
from emtest import EmTest
from buttons import press
from leds import Leds
import pytest
import time
import random


class RGBStateMachine:

    def __init__(self):
        self.red = False
        self.green = False
        self.blue = False
        self.index = 0

    def next(self):
        self.index += 1
        self.index %= 3

    def switch(self):
        match self.index:
            case 0:
                self.red = not self.red
            case 1:
                self.green = not self.green
            case 2:
                self.blue = not self.blue

    def test(self, leds: Leds):
        assert leds.red() == pytest.approx(1.0 if self.red else 0.0)
        assert leds.green() == pytest.approx(1.0 if self.green else 0.0)
        assert leds.blue() == pytest.approx(1.0 if self.blue else 0.0)


class TestGPIO(EmTest):
    def setup_class(self):
        self.replacements = []
        self.week_branch = "Week_02"
        super().setup_class(self)

    def led_switch(self, led_val: Callable):
        assert led_val() == pytest.approx(0.0)
        press(self.buttons.sw3)
        assert led_val() == pytest.approx(1.0)
        time.sleep(0.5)
        press(self.buttons.sw3)
        assert led_val() == pytest.approx(0.0)

    def test_red_switching(self):
        self.led_switch(self.leds.red)

    def test_green_switching(self):
        press(self.buttons.sw2)
        self.led_switch(self.leds.green)

    def test_blue_switching(self):
        press(self.buttons.sw2)
        time.sleep(0.5)
        press(self.buttons.sw2)
        self.led_switch(self.leds.blue)

    def test_button_response_only_once(self):
        self.buttons.sw3.on()
        time.sleep(1)
        self.buttons.sw3.off()
        assert self.leds.red() == pytest.approx(1.0)
        time.sleep(0.5)
        self.buttons.sw3.on()
        time.sleep(1.3)
        self.buttons.sw3.off()
        assert self.leds.red() == pytest.approx(0.0)
        time.sleep(0.5)
        self.buttons.sw3.on()
        time.sleep(1.5)
        self.buttons.sw3.off()
        assert self.leds.red() == pytest.approx(1.0)

    @pytest.mark.skip(reason="debouncing removal")
    def test_switch_pulse_detection(self):
        self.buttons.sw3.pulse()
        time.sleep(0.5)
        assert self.leds.red() == pytest.approx(0.0)



    @pytest.mark.skip(reason="debouncing removal")
    def test_next_pulse_detection(self):
        self.buttons.sw2.pulse()
        time.sleep(0.5)
        self.buttons.sw2.reset()
        self.led_switch(self.leds.red)

    def state_machine_iteration(self, state_machine: RGBStateMachine):
        action_choice = random.randrange(0, 4)
        state_machine.test(self.leds)
        match action_choice:
            case 0:
                self.buttons.sw3.on()
                time.sleep(0.5)
                self.buttons.sw3.off()
                state_machine.switch()
            case 1:
                self.buttons.sw2.on()
                time.sleep(0.5)
                self.buttons.sw2.off()
                state_machine.next()
            #case 2:
            #    self.buttons.sw3.pulse()
            #    time.sleep(0.5)
            #    self.buttons.sw3.reset()
            #case 3:
            #    self.buttons.sw2.pulse()
            #    time.sleep(0.5)
            #    self.buttons.sw2.reset()

    def test_random_state_testing(self):
        state_machine = RGBStateMachine()
        random.seed(42)
        for _ in range(50):
            self.state_machine_iteration(state_machine)
