import pytest

from emtest import EmTest
import colorsys
import time


def rgb_to_hsv_normalized(rgb: (float, float, float)) -> (int, int, int):
    r, g, b = rgb
    h, s, v = colorsys.rgb_to_hsv(r, g, b)
    return int(h * 256), int(s * 256), int(v * 256)


class TestHSV(EmTest):
    def setup_class(self):
        self.replacements = []
        self.week_branch = "Week_04"
        super().setup_class(self)

    def test_hsv_cycle(self):
        """
        Effectively, we check if the next RGB value corresponds to higher hue with some rollover protection
        it will need live testing and we expect that we will need to make adjustments
        """
        # Chosen by testing certain amount of values
        sample_count = 4_000
        blue_hue_value = 165
        time.sleep(0.5)
        prev_hsv = rgb_to_hsv_normalized(self.leds.get_led_values(sample_count))
        rollovers = 0
        for _ in range(500):
            next_hsv = rgb_to_hsv_normalized(self.leds.get_led_values(sample_count))
            prev_h, _, _ = prev_hsv
            next_h, _, _ = next_hsv
            # Rollover protection
            if next_h <= 80 and prev_h > next_h and prev_h >= blue_hue_value:
                rollovers += 1
            else:
                assert prev_h - 80 <= next_h
            prev_hsv = next_hsv
        # We expect at least four rollovers
        assert rollovers >= 4

    @pytest.mark.skip(reason="interrupt use not mandatory")
    def test_interrupts_used(self):
        assert self.interrupts.pit.has_any_interrupt_set()
