from emtest import EmTest
import serial
import pytest
import time
import random


def color_approx_equal(sampled_color: float, tested_color: float, abs_tol: float) -> bool:
    return abs(sampled_color - tested_color) < abs_tol


def rgb_color_test(sampled_colors: (float, float, float), tested_colors: (float, float, float), abs_tol) -> bool:
    red_test = color_approx_equal(sampled_colors[0], tested_colors[0], abs_tol)
    green_test = color_approx_equal(sampled_colors[1], tested_colors[1], abs_tol)
    blue_test = color_approx_equal(sampled_colors[2], tested_colors[2], abs_tol)
    print("Sampled: " + str(sampled_colors) + " tested: " + str(tested_colors) + "results: " + str((red_test, green_test, blue_test)))
    return red_test and green_test and blue_test


class TestUART(EmTest):
    def setup_class(self):
        self.week_branch = "Week_08"
        self.replacements = []
        super().setup_class(self)

    def test_full_light(self):
        full_light_data = bytes([255, 255, 255])
        time.sleep(0.5)
        self.uart.write(full_light_data)
        self.uart.flush()
        assert self.leds.red() == pytest.approx(1.0, abs=0.2)
        assert self.leds.green() == pytest.approx(1.0, abs=0.2)
        assert self.leds.blue() == pytest.approx(1.0, abs=0.2)

    def test_random_data(self):
        time.sleep(0.5)
        random.seed(42)
        failed_pwm = 0
        iterations = 50
        for _ in range(iterations):
            red = random.randint(0, 255)
            green = random.randint(0, 255)
            blue = random.randint(0, 255)
            packet = bytes([red, green, blue])
            self.uart.write(packet)
            self.uart.flush()
            time.sleep(0.5)
            red_pwm, green_pwm, blue_pwm = self.leds.get_led_values()
            red = red / 255
            green = green / 255
            blue = blue / 255
            if not rgb_color_test((red_pwm, green_pwm, blue_pwm), (red, green, blue), 0.2):
                failed_pwm += 1
        # Note: This means most pwm samples deviated too much from the sent values
        assert (failed_pwm / iterations) <= 0.1
